/*---------------------------------------------------------------------------*/
/* GIVE-DEST.C -- contains routines to figure out what the destination       */
/*   conditions look like.                                                   */

#include <uaidef.h>
#include <ssdef.h>
#include <fibdef.h>
#include <string.h>
#include <descrip.h>
#include <iodef.h>
#include "give.h"
#include "give-dest.h"
#include "give-uai.h"

struct quota_file_transfer_block {
   ulong flags;
   ulong UIC;
   ulong usage;
   ulong permanent;
   ulong overdraft;
   ulong unused[3];
};

struct quota_descriptor {
   ulong size;
   void *pointer;
};

/*---------------------------------------------------------------------------*/
/* Get_Destination_Directory -- this function constructs the destination     */
/*  directory for the files out of the destination username's login          */
/*  diretory.                                                                */

ulong Get_Destination_Directory( char *dest_username, char *dest_directory )
{
   ulong ss_stat;
   char dest_login_device_bl[K_DEVICE_LEN+1+1];
   char *dest_login_device = dest_login_device_bl+1;
   char dest_login_directory_bl[K_FILENAME_LEN+1+1];
   char *dest_login_directory = dest_login_directory_bl+1;

/*------*/
/* Get the device name.                                                      */

   ss_stat = Get_Single_UAI( dest_username, UAI$_DEFDEV,
                             sizeof( dest_login_device_bl ),
                             dest_login_device_bl,
                             NULL );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( dest_login_device, dest_login_device_bl[0] );

/*------*/
/* Get the directory name.                                                   */

   ss_stat = Get_Single_UAI( dest_username, UAI$_DEFDIR,
                             sizeof( dest_login_directory_bl ),
                             dest_login_directory_bl,
                             NULL );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( dest_login_directory, dest_login_directory_bl[0] );

/*------*/
/* Construct the login directory of the person in the supplied string        */

   strcpy( dest_directory, dest_login_device );
   strcat( dest_directory, dest_login_directory );

   return( SS$_NORMAL );

}

/*---------------------------------------------------------------------------*/
/* Get_Destination_Diskquota -- this function determines the diskquota that  */
/*  the destination user has remaining.                                      */

ulong Get_Destination_Diskquota( char *dest_directory, char *dest_username,
                                 ulong *dest_diskquota )
{
   ulong ss_stat;
   struct quota_file_transfer_block QFTB;
   struct fibdef1 fib;
   ushort chan;
   struct quota_descriptor fib_dsc;
   struct quota_descriptor QFTB_dsc;
   ulong iosb[2];
   ulong return_length;
   size_t device_length;
   $DESCRIPTOR_Z( dest_device_d, dest_directory );
   ulong i;

/*------*/
/* Get the UIC of the destination user                                       */

   ss_stat = Get_Single_UAI( dest_username, UAI$_UIC,
                             sizeof( QFTB.UIC ),
                             &(QFTB.UIC),
                             NULL );
   if( ss_failure( ss_stat ) )
      return( ss_stat );


/*------*/
/* Get the device name set up as a descriptor                                */

   device_length = strcspn( dest_directory, ":" );
   dest_device_d.dsc$w_length = device_length;

/*------*/
/* Set up the various descriptors                                            */

   fib_dsc.size = sizeof( fib );
   fib_dsc.pointer = &fib;
   QFTB_dsc.size = sizeof( QFTB );
   QFTB_dsc.pointer = &QFTB;

/*------*/
/* Open a channel to the device                                              */

   ss_stat = sys$assign( &dest_device_d, &chan, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Set up the FIB for the QIO function                                       */

   for( i=0; i<sizeof( fib ); i++ )
      ((char *)(&fib))[i] = '\0';

   fib.fib$w_cntrlfunc = FIB$C_EXA_QUOTA;

/*------*/
/* Do the QIO to get the disk quota                                          */

   iosb[0] = iosb[1] = 0;
   ss_stat = sys$qiow( 0, chan, IO$_ACPCONTROL, &iosb, 0, 0, &fib_dsc, 
                    &QFTB_dsc, &return_length, &QFTB_dsc, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   if( ss_failure( iosb[0] ) )
      return( iosb[0] );

/*------*/
/* Clean up a bit and exit                                                   */

   ss_stat = sys$dassgn( chan );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   if( QFTB.permanent < QFTB.usage )
      *dest_diskquota = 0;
   else
      *dest_diskquota = QFTB.permanent - QFTB.usage;
   return( SS$_NORMAL );
  
}

