/*---------------------------------------------------------------------------*/
/* GIVE-LOG.C -- contains logging routines for GIVE.C                        */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <ssdef.h>
#include <starlet.h>
#include <brkdef.h>
#include <jpidef.h>
#include <str$routines.h>
#include <strdef.h>
#include <lckdef.h>
#include "give.h"
#include "give-log.h"
#include "give-jpi.h"

/*---------------------------------------------------------------------------*/
/* Log_Give -- this function logs the file transfer in a text file for the   */
/* destination user.                                                         */

ulong Log_Give( char *directory, char *username, char *filename )
{
   ulong ss_stat;
   FILE *log_fp;
   char log_filename[K_FILENAME_LEN+1];
   char current_time[K_TIMESTR_LEN+1];
   $DESCRIPTOR( current_time_d, current_time );
   char from_username[K_USERNAME_LEN+1];
   $DESCRIPTOR( from_username_d, from_username );
   char hold_string[256];
   $DESCRIPTOR( hold_string_d, hold_string );
   char resname[K_RESOURCE_LEN+1];
   $DESCRIPTOR( resname_d, resname );
   ulong lksb[2];

   strcpy( log_filename, directory );
   strcat( log_filename, K_LOG_FILENAME );

   strcpy( resname, "GIVELOG-" );
   strcat( resname, username );
   resname_d.dsc$w_length = strlen( resname );

   ss_stat = sys$enqw( 0, LCK$K_EXMODE, lksb, LCK$M_SYSTEM, &resname_d,
                       0, 0, 0, 0, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   if( (log_fp = fopen( log_filename, "a" ))==NULL )
      return( GIVE_LOG_OPENERR );

   ss_stat = sys$asctim( &(current_time_d.dsc$w_length), &current_time_d,
                         0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( current_time, current_time_d.dsc$w_length );

   ss_stat = Get_Single_JPI( 0, JPI$_USERNAME, K_USERNAME_LEN, hold_string,
                             &(hold_string_d.dsc$w_length));
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   ss_stat = str$trim( &from_username_d, &hold_string_d,
                       &(from_username_d.dsc$w_length) );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( from_username, from_username_d.dsc$w_length );

   fprintf( log_fp, "%-23s: user %-12s gave %s\n", current_time,
            from_username, filename );

   fclose( log_fp );

   ss_stat = sys$deq( lksb[1], 0, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   return( SS$_NORMAL );
}

/*---------------------------------------------------------------------------*/
/* Log_Broadcast -- this function broadcasts a message informing the         */
/* destination user about the file transfer.                                 */

ulong Log_Broadcast( char *to_username, char *filename, ulong size )
{
   ulong ss_stat;

   char broadcast_buff[132+1];
   $DESCRIPTOR( broadcast_buff_d, broadcast_buff );
   char current_time[K_TIMESTR_LEN+1];
   $DESCRIPTOR( current_time_d, current_time );
   char from_username[K_USERNAME_LEN+1];
   $DESCRIPTOR( from_username_d, from_username );
   $DESCRIPTOR_Z( to_username_d, to_username );
   char hold_string[256];
   $DESCRIPTOR( hold_string_d, hold_string );

   ss_stat = sys$asctim( &(current_time_d.dsc$w_length), &current_time_d,
                         0, 1 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( current_time, current_time_d.dsc$w_length );

   ss_stat = Get_Single_JPI( 0, JPI$_USERNAME, K_USERNAME_LEN, hold_string,
                             &(hold_string_d.dsc$w_length));
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   ss_stat = str$trim( &from_username_d, &hold_string_d,
                       &(from_username_d.dsc$w_length) );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   null_terminate( from_username, from_username_d.dsc$w_length );

   sprintf( broadcast_buff, "%c%s (%d blocks) received from %s at %s", K_BELL,
            filename, size, from_username, current_time );
   broadcast_buff_d.dsc$w_length = strlen( broadcast_buff );

   ss_stat = sys$brkthruw( 0, &broadcast_buff_d, &to_username_d,
                           BRK$C_USERNAME, 0, 32, BRK$M_CLUSTER,
                           BRK$C_MAIL, 8, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   return( SS$_NORMAL );
}
