/*---------------------------------------------------------------------------*/
/* GIVE.C -- This program allows the unprivileged user to copy an arbitrary  */
/*   file to a user who has given him permission to do so.                   */
/*---------------------------------------------------------------------------*/

#include <jpidef.h>
#include <ssdef.h>
#include <descrip.h>
#include <uaidef.h>
#include <stdio.h>
#include <prvdef.h>
#include <rms.h>

#include "give.h"
#include "give-cli.h"
#include "give-dest.h"
#include "give-auth.h"
#include "give-rms.h"
#include "give-log.h"
#include "give-prv.h"

/*------*/
/* Constant declarations                                                     */

#define K_USERNAME_PARM "USERNAME"
#define K_FILENAME_PARM "FILENAME"
#define K_SETUP_QUAL "SETUP"
#define K_DEBUG_QUAL "DEBUG"
#define K_CHECK_QUAL "CHECK"
#define K_LOG_QUAL "LOG"

/*------*/
/* Globals                                                                   */

boolean qual_debug;
boolean qual_log;

ulong Give_One_File( char *, char * );
void Setup( void );
void Print_Usage( void );
void Check( void );

/*---------------------------------------------------------------------------*/
/* main -- the usual                                                         */

int main( int argc, char **argv, char **envp ) {

   ulong ss_stat;
   ulong i, j;
   ulong wild_count;
   char *parms_username[K_MAX_USERNAMES+1];
   char *parms_filename[K_MAX_FILENAMES+1];
   boolean qual_setup;
   boolean qual_check;
   char *curr_username;
   char *curr_filename;
   char esa[NAM$C_MAXRSS+1];
   char rsa[NAM$C_MAXRSS+1];
   struct FAB fab;
   struct NAM nam;
   ulong message_vect[25];

/*------*/
/* Disable installed privileges                                              */

   Priv_Disable( PRV$M_BYPASS | PRV$M_WORLD | PRV$M_OPER | PRV$M_SYSLCK );

/*------*/
/* Get the command line arguments                                            */


   Get_Presence( K_SETUP_QUAL, &qual_setup );
   Get_Presence( K_CHECK_QUAL, &qual_check );
   Get_Presence( K_LOG_QUAL, &qual_log );
/*   Get_Presence( K_DEBUG_QUAL, &qual_debug ); */
   qual_debug = FALSE;
   ss_stat = Get_List( K_USERNAME_PARM, parms_username, 255 );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   ss_stat = Get_List( K_FILENAME_PARM, parms_filename, 255 );

   if( qual_setup ) {
      Setup();
      exit( SS$_NORMAL );
   }
   if( qual_check ) {
      Check();
      exit( SS$_NORMAL );
   }

   if( parms_filename[0][0] == '\0' ) {
      Print_Usage();
      exit( GIVE_NOFILENAME );
   }
   if( parms_username[0][0] == '\0' ) {
      Print_Usage();
      exit( GIVE_NOUSERNAME );
   }

   for( i=0; parms_username[i][0] != '\0'; i++ ) {
      for( j=0; parms_filename[j][0] != '\0'; j++ ) {

         fab = cc$rms_fab;
         nam = cc$rms_nam;
   
         fab.fab$l_fna = parms_filename[j];
         fab.fab$b_fns = strlen( parms_filename[j] );
         fab.fab$l_nam = &nam;
         nam.nam$l_esa = esa;
         nam.nam$b_ess = NAM$C_MAXRSS;
         nam.nam$l_rsa = rsa;
         nam.nam$b_rss = NAM$C_MAXRSS;
   
         ss_stat = sys$parse( &fab, 0, 0 );
         if( ss_failure( ss_stat ) ) {
            ERR_Skipping( parms_filename[j], ss_stat );
            continue;
         }

         wild_count = 0;
         ss_stat = sys$search( &fab, 0, 0 );
         while( ss_success( ss_stat ) ) {
            wild_count++;
            null_terminate( rsa, nam.nam$b_rsl );
            ss_stat = Give_One_File( parms_username[i], rsa );
            if( ss_failure( ss_stat ) )
               ERR_NotGiven( rsa, ss_stat );
            else
               MSG_Given( parms_username[i], rsa );
            ss_stat = sys$search( &fab, 0, 0 );
         }
         if( wild_count == 0 )
            ERR_Skipping( parms_filename[j], ss_stat );

      }
   }

}

/*---------------------------------------------------------------------------*/
/* Give_One_File -- attempts to give a single file.                          */

ulong Give_One_File( char *spec_username, char *spec_filename )
{
   ulong ss_stat;
   char destination_directory[K_FILENAME_LEN+1];
   char login_directory[K_FILENAME_LEN+1];
   char destination_filename[K_FILENAME_LEN+1];
   char destination[K_FILENAME_LEN+1];
   char logfile_name[K_FILENAME_LEN+1];
   $DESCRIPTOR( destination_d, destination );
   ulong destination_diskquota;
   ulong source_file_size;

/*------*/
/* Get destination information                                               */

   Priv_Enable( PRV$M_BYPASS );
   ss_stat = Get_Destination_Directory( spec_username, login_directory );
   Priv_Disable( PRV$M_BYPASS );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   strcpy( destination_directory, login_directory );

   if( qual_debug )
      printf( "Destination directory: %s\n", destination_directory );

   Priv_Enable( PRV$M_BYPASS );
   ss_stat = Get_Destination_Diskquota( destination_directory,
                                        spec_username,
                                        &destination_diskquota );
   Priv_Disable( PRV$M_BYPASS );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   if( qual_debug )
      printf( "Destination diskquota: %d\n", destination_diskquota );

/*------*/
/* Get source file information                                               */

   ss_stat = Get_File_Size( spec_filename, &source_file_size );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   if( qual_debug )
      printf( "Source file size: %d\n", source_file_size );

/*------*/
/* Get authorization information                                             */

   strcpy( destination_filename, spec_filename );
   Priv_Enable( PRV$M_BYPASS );
   ss_stat = Check_Authorization( destination_directory, source_file_size,
                                  destination_diskquota,
                                  destination_filename );
   Priv_Disable( PRV$M_BYPASS );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   if( qual_debug )
      printf( "Destination filename: %s%s.\n", destination_directory,
              destination_filename );

/*------*/
/* Attempt to create the destination directory.                              */

   strcpy( destination, destination_directory );
   destination_d.dsc$w_length = strlen( destination );

   Priv_Enable( PRV$M_BYPASS );
   ss_stat = lib$create_dir( &destination_d, 0, 0, 0, 0, 0 );
   Priv_Disable( PRV$M_BYPASS );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Copy the file to the destination directory.                               */

   strcat( destination, destination_filename );

   Priv_Enable( PRV$M_BYPASS );
   ss_stat = Copy_File( spec_filename, destination );
   Priv_Disable( PRV$M_BYPASS );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Append an informative message to the log file.                            */

   Priv_Enable( PRV$M_BYPASS | PRV$M_SYSLCK );
   ss_stat = Log_Give( login_directory, spec_username, destination );
   Priv_Disable( PRV$M_BYPASS | PRV$M_SYSLCK );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Send a broadcast message to the destination username.                     */

   Priv_Enable( PRV$M_WORLD | PRV$M_OPER );
   ss_stat = Log_Broadcast( spec_username, destination, source_file_size );
   Priv_Disable( PRV$M_WORLD | PRV$M_OPER );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

}

/*---------------------------------------------------------------------------*/
/* Setup for receiving.                                                      */

void Setup( void )
{
   ulong ss_stat;
   char hold_string[256];
   $DESCRIPTOR( hold_string_d, hold_string );
   char my_username[K_USERNAME_LEN+1];
   $DESCRIPTOR( my_username_d, my_username );
   char login_directory[K_FILENAME_LEN+1];
   char destination_directory[K_FILENAME_LEN+1];
   $DESCRIPTOR( destination_directory_d, destination_directory );
   char auth_filename[K_FILENAME_LEN+1];
   
/*------*/
/* Get our username.                                                         */

   ss_stat = Get_Single_JPI( 0, JPI$_USERNAME, K_USERNAME_LEN, hold_string,
                             &(hold_string_d.dsc$w_length));
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   ss_stat = str$trim( &my_username_d, &hold_string_d,
                       &(my_username_d.dsc$w_length) );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   null_terminate( my_username, my_username_d.dsc$w_length );
   
/*------*/
/* Get our login directory.                                                  */

   ss_stat = Get_Destination_Directory( my_username, login_directory );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

/*------*/
/* Create a sample authorization file.                                       */

   strcpy( auth_filename, login_directory );
   strcat( auth_filename, K_AUTH_FILENAME );

   ss_stat = Copy_File( K_SAMPLE_AUTH_FILENAME, auth_filename );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   MSG_AuthCreated( auth_filename );

/*------*/
/* Create spool directory.                                                   */

   strcpy( destination_directory, login_directory );
   destination_directory[strlen(destination_directory)-1] = '\0';
   strcat( destination_directory, K_DESTDIR_POSTFIX );
   destination_directory_d.dsc$w_length = strlen( destination_directory );

   ss_stat = lib$create_dir( &destination_directory_d, 0, 0, 0, 0, 0 );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   MSG_DirCreated( destination_directory );

}

/*---------------------------------------------------------------------------*/
/* Print_Usage -- prints a usage message.                                    */

void Print_Usage( void )
{
   puts( "GIVE -- user-to-user file transfer utility\n" );
   puts( "Usage:  GIVE filename username  (to transfer a file)" );
   puts( "        GIVE/SETUP              (to set up GIVE for receiving)" );
   puts( "        GIVE/CHECK              (to verify authorization file)\n" );
}

/*---------------------------------------------------------------------------*/
/* Check -- check authorization file for consistency                         */

void Check( void )
{
   ulong ss_stat;
   char hold_string[256];
   $DESCRIPTOR( hold_string_d, hold_string );
   char my_username[K_USERNAME_LEN+1];
   $DESCRIPTOR( my_username_d, my_username );
   char login_directory[K_FILENAME_LEN+1];
   char auth_filename[K_FILENAME_LEN+1];
   struct auth_entry *auth_head;

/*------*/
/* Get our username.                                                         */

   ss_stat = Get_Single_JPI( 0, JPI$_USERNAME, K_USERNAME_LEN, hold_string,
                             &(hold_string_d.dsc$w_length));
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   ss_stat = str$trim( &my_username_d, &hold_string_d,
                       &(my_username_d.dsc$w_length) );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   null_terminate( my_username, my_username_d.dsc$w_length );
   
/*------*/
/* Get our login directory.                                                  */

   ss_stat = Get_Destination_Directory( my_username, login_directory );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

/*------*/
/* Get the authorization filename.                                           */

   strcpy( auth_filename, login_directory );
   strcat( auth_filename, K_AUTH_FILENAME );

/*------*/
/* Attempt to read the authorization file.                                   */

   ss_stat = Read_Authorization_File( auth_filename, &auth_head );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   MSG_CheckOK( auth_filename );
   exit( SS$_NORMAL );   
}
